==> OpenCL-0.8.5.tgz: OpenCL-0.8.5/src/dl_parm.cpp <==
/*************************************************
* Discrete Logarithm Parameters Source File      *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/dl_parm.h>
#include <opencl/primes.h>
#include <opencl/numthry.h>

namespace OpenCL {

/*************************************************
* DL_Group Constructor               *
*************************************************/
DL_Group::DL_Group(u32bit pbits, PrimeType type)
   {
   if(pbits < 64)
      throw Invalid_Argument("DL_Group: prime size too small");

   if(type == Strong)
      safe_prime = true;
   else
      safe_prime = false;

   if(type == Strong)
      {
      p = random_safe_prime(pbits);
      q = (p - 1) / 2;
      g = 2;
      }
   else if(type == Prime_Subgroup || type == DSA_Kosherizer)
      {
      if(type == Prime_Subgroup)
         {
         u32bit qbits = 2 * dl_work_factor(pbits);
         q = random_prime(qbits);
         BigInt q_2 = 2 * q;
         do
            p = random_integer(pbits - qbits - 1) * q_2 + 1;
         while(p.bits() != pbits || !is_prime(p));
         }
      else
         generate_dsa_primes(p, q, pbits);

      BigInt e = (p - 1) / q;
      u32bit prime_no = 0;
      do
         g = power_mod(PRIMES[prime_no++], e, p);
      while(g == 1);
      }
   initialized = true;
   }

/*************************************************
* DL_Group Constructor               *
*************************************************/
DL_Group::DL_Group(const BigInt& p1, const BigInt& g1)
   {
   if(p1 < 3 || g1 <= 1 || g1 >= p1)
      throw Invalid_Argument("DL parameters: Invalid prime or generator");

   p = p1;
   g = g1;
   q = (p1 - 1) / 2;
   safe_prime = true;
   initialized = true;
   }

/*************************************************
* DL_Group Constructor               *
*************************************************/
DL_Group::DL_Group(const BigInt& p1, const BigInt& q1,
                                           const BigInt& g1)
   {
   if(p1 < 3 || q1 < 3 || q1 >= p1 || g1 <= 1 || g1 >= p1)
      throw Invalid_Argument("DL parameters: Invalid prime(s) or generator");

   p = p1;
   g = g1;
   q = q1;

   if((p - 1) / 2 == q1)
      safe_prime = true;
   else
      safe_prime = false;
   initialized = true;
   }

/*************************************************
* DL_Group Constructor               *
*************************************************/
DL_Group::DL_Group()
   {
   initialized = safe_prime = false;
   }

/*************************************************
* Verify the parameters                          *
*************************************************/
bool DL_Group::verify() const
   {
   if(!initialized)
      throw Exception("DL_Group: Was never initialized");
   if(!verify_prime(p) || !verify_prime(q))
      return false;
   if((p - 1) % q != 0)
      return false;
   return true;
   }

/*************************************************
* Return the prime                               *
*************************************************/
const BigInt& DL_Group::get_p() const
   {
   if(!initialized)
      throw Exception("DL_Group: Was never initialized");
   return p;
   }

/*************************************************
* Return the subgroup                            *
*************************************************/
const BigInt& DL_Group::get_q() const
   {
   if(!initialized)
      throw Exception("DL_Group: Was never initialized");
   return q;
   }

/*************************************************
* Return the generator                           *
*************************************************/
const BigInt& DL_Group::get_g() const
   {
   if(!initialized)
      throw Exception("DL_Group: Was never initialized");
   return g;
   }

/*************************************************
* See if the prime is st p = 2*q + 1, q prime    *
*************************************************/
bool DL_Group::prime_is_safe() const
   {
   if(!initialized)
      throw Exception("DL_Group: Was never initialized");
   return safe_prime;
   }

}

==> OpenCL-0.8.7.tgz: OpenCL-0.8.7/src/dl_parm.cpp <==
/*************************************************
* Discrete Logarithm Parameters Source File      *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/dl_parm.h>
#include <opencl/primes.h>
#include <opencl/numthry.h>

namespace OpenCL {

/*************************************************
* DL_Group Constructor                           *
*************************************************/
DL_Group::DL_Group(u32bit pbits, PrimeType type)
   {
   if(pbits < 64)
      throw Invalid_Argument("DL_Group: prime size too small");

   if(type == Strong)
      safe_prime = true;
   else
      safe_prime = false;

   if(type == Strong)
      {
      p = random_safe_prime(pbits);
      q = (p - 1) / 2;
      g = 2;
      }
   else if(type == Prime_Subgroup || type == DSA_Kosherizer)
      {
      if(type == Prime_Subgroup)
         {
         u32bit qbits = 2 * dl_work_factor(pbits);
         q = random_prime(qbits);
         BigInt q_2 = 2 * q;
         do
            p = random_integer(pbits - qbits - 1) * q_2 + 1;
         while(p.bits() != pbits || !is_prime(p));
         }
      else
         generate_dsa_primes(p, q, pbits);

      BigInt e = (p - 1) / q;
      u32bit prime_no = 0;
      do
         g = power_mod(PRIMES[prime_no++], e, p);
      while(g == 1);
      }
   }


/*************************************************
* DL_Group Constructor                           *
*************************************************/
DL_Group::DL_Group(const SecureVector<byte>& seed, u32bit pbits, u32bit start)
   {
   if(!generate_dsa_primes(p, q, seed, seed.size(), pbits, start))
      throw Invalid_Argument("DL_Group: The seed/counter given does not "
                             "generate a DSA group");

   BigInt e = (p - 1) / q;
   u32bit prime_no = 0;
   do
      g = power_mod(PRIMES[prime_no++], e, p);
   while(g == 1);
   }

/*************************************************
* DL_Group Constructor                           *
*************************************************/
DL_Group::DL_Group(const BigInt& p1, const BigInt& g1)
   {
   if(p1 < 3 || g1 <= 1 || g1 >= p1)
      throw Invalid_Argument("DL parameters: Invalid prime or generator");

   p = p1;
   g = g1;
   q = (p1 - 1) / 2;
   safe_prime = true;
   }

/*************************************************
* DL_Group Constructor                           *
*************************************************/
DL_Group::DL_Group(const BigInt& p1, const BigInt& q1, const BigInt& g1)
   {
   if(p1 < 3 || q1 < 3 || q1 >= p1 || g1 <= 1 || g1 >= p1)
      throw Invalid_Argument("DL parameters: Invalid prime(s) or generator");

   p = p1;
   g = g1;
   q = q1;

   if((p - 1) / 2 == q1)
      safe_prime = true;
   else
      safe_prime = false;
   }

/*************************************************
* Verify the parameters                          *
*************************************************/
bool DL_Group::verify() const
   {
   if(!verify_prime(p) || !verify_prime(q))
      return false;
   if((p - 1) % q != 0)
      return false;
   return true;
   }

/*************************************************
* Return the prime                               *
*************************************************/
const BigInt& DL_Group::get_p() const
   {
   return p;
   }

/*************************************************
* Return the subgroup                            *
*************************************************/
const BigInt& DL_Group::get_q() const
   {
   return q;
   }

/*************************************************
* Return the generator                           *
*************************************************/
const BigInt& DL_Group::get_g() const
   {
   return g;
   }

/*************************************************
* See if the prime is st p = 2*q + 1, q prime    *
*************************************************/
bool DL_Group::prime_is_safe() const
   {
   return safe_prime;
   }

}
